//
//  main.swift
//  
//
//  Created by Thibaut Schmitt on 20/12/2021.
//

import Foundation
import CLIToolCore
import ArgumentParser

enum ColorStyle: String, Decodable {
    case light
    case all
}

struct ColorTool: ParsableCommand {
    static let defaultExtensionName = "UIColor"
    static let assetsColorsFolderName = "Colors"
    
    @Flag(name: .customShort("f"), help: "Should force generation")
    var forceGeneration = false
    
    @Argument(help: "Input files where colors ared defined.")
    var inputFile: String
    
    @Option(help: "Color style to generate: light for light colors only, or all for dark and light colors")
    var style: String
    
    @Option(help: "Path of xcassets where to generate colors")
    var xcassetsPath: String
    
    @Option(help: "Path where to generate the extension.")
    var extensionOutputPath: String
    
    @Option(help: "Extension name. If not specified, it will generate an UIFont extension")
    var extensionName: String = Self.defaultExtensionName
    
    @Option(help: "Extension suffix. Ex: MyApp, it will generate {extensionName}+ColorsMyApp.swift")
    var extensionSuffix: String = ""
    
    var colorStyle: ColorStyle { ColorStyle(rawValue: style) ?? .all }
    var extensionFileName: String { "\(extensionName)+Color\(extensionSuffix).swift" }
    var extensionFilePath: String { "\(extensionOutputPath)/\(extensionFileName)" }
    
    public func run() throws {
        print("[ColorTool] Starting colors generation")
        
        print("[ColorTool] Will use inputFile \(inputFile) to generate \(colorStyle) colors in xcassets \(xcassetsPath)")
        print("[ColorTool] Extension will be \(extensionFilePath)")
        

        // Check if needed to regenerate
        guard GeneratorChecker.shouldGenerate(force: forceGeneration, inputFilePath: inputFile, extensionFilePath: extensionFilePath) else {
            print("[ColorTool] Colors are already up to date :) ")
            return
        }
        print("[ColorTool] Will generate colors")
        
        // Check if Xcasset exists
        let fileManager = FileManager()
        guard fileManager.fileExists(atPath: xcassetsPath) else {
            ColorTool.exit(withError: ColorToolError.fileNotExists(xcassetsPath))
        }
        
        // Delete current colors
        Shell.shell("rm", "-rf", "\(xcassetsPath)/Colors/*")
        
        // Get colors
        let colorsToGen = getColorsGen()
        
        // Generate all colors in xcassets
        let colorAssetHelper = ColorXcassetHelper(xcassetsPath: xcassetsPath, colors: colorsToGen)
        colorAssetHelper.generateXcassetColors()
        
        // Generate extension
        let extensionGenerator = ColorExtensionGenerator(colors: colorsToGen,
                                                         extensionClassname: extensionName,
                                                         isUIColorExtension: isUIColorExtension())
        let extensionHeader = extensionGenerator.getHeader()
        let extensionProperties = extensionGenerator.getProperties()
        let extensionFooter = extensionGenerator.getFooter()
        
        generateExtensionFile(extensionHeader, extensionProperties, extensionFooter)
        
        print("[ColorTool] Colors generated")
    }
    
    private func generateExtensionFile(_ args: String...) {
        // Create file if not exists
        let fileManager = FileManager()
        if fileManager.fileExists(atPath: extensionFilePath) == false {
            Shell.shell("touch", "\(extensionFilePath)")
        }
        
        // Create extension content
        let extensionContent = args.joined(separator: "\n")
        
        // Write content
        let extensionFilePathURL = URL(fileURLWithPath: extensionFilePath)
        do {
            try extensionContent.write(to: extensionFilePathURL, atomically: true, encoding: .utf8)
        } catch (let error) {
            ColorTool.exit(withError: ColorToolError.writeExtension(extensionFilePath, error.localizedDescription))
        }
    }
    
    private func getColorsGen() -> [GenColor] {
        // Get content of input file
        let inputFileContent = try! String(contentsOfFile: inputFile, encoding: .utf8)
        let colorsByLines = inputFileContent.components(separatedBy: .newlines)
        
        // Iterate on each line of input file
        return colorsByLines.enumerated().compactMap { lineNumber, colorLine in
            // Required format:
            // colorName="#RGB/#ARGB", colorName "#RGB/#ARGB", colorName "#RGB/#ARGB" "#RGB/#ARGB"
            let colorLineCleanedUp = colorLine
                .removeTrailingWhitespace()
                .replacingOccurrences(of: "=", with: "") // Keep compat with current file format
            
            guard colorLineCleanedUp.hasPrefix("#") == false, colorLineCleanedUp.isEmpty == false else {
                print("[ColorTool] ⚠️  BadFormat or empty line (line number: \(lineNumber + 1)). Skip this line")
                return nil
            }
            
            let colorContent = colorLineCleanedUp.split(separator: " ")
            
            guard colorContent.count >= 2 else {
                ColorTool.exit(withError: ColorToolError.badFormat(colorLine))
            }
            
            switch colorStyle {
            case .light:
                return GenColor(name: String(colorContent[0]), light: String(colorContent[1]), dark: String(colorContent[1]))
                
            case .all:
                if colorContent.count == 3 {
                    return GenColor(name: String(colorContent[0]), light: String(colorContent[1]), dark: String(colorContent[2]))
                }
                return GenColor(name: String(colorContent[0]), light: String(colorContent[1]), dark: String(colorContent[1]))
            }
        }
    }
    
    // MARK: - Helpers
    
    private func isUIColorExtension() -> Bool {
        extensionName == Self.defaultExtensionName
    }
}

ColorTool.main()

/*
 Command samples:
 
 1. UIColor extension without suffix
 swift run -c release ColorToolCore -f ./SampleFiles/Colors/sampleColors1.txt --style all --xcassets-path "./SampleFiles/Colors/colors.xcassets" --extension-output-path "./SampleFiles/Colors/Generated/" --extension-name "UIColor"
 
 2. UIColor extension with custom suffix
 swift run -c release ColorToolCore -f ./SampleFiles/Colors/sampleColors1.txt --style all --xcassets-path "./SampleFiles/Colors/colors.xcassets" --extension-output-path "./SampleFiles/Colors/Generated/" --extension-name "UIColor" --extension-suffix "SampleApp"
 
 3. Custom extension with only light theme colors (R2Color)
 swift run -c release ColorToolCore -f ./SampleFiles/Colors/sampleColors1.txt --style light --xcassets-path "./SampleFiles/Colors/colors.xcassets" --extension-output-path "./SampleFiles/Colors/Generated/" --extension-name "R2Color"
 */
